defmodule Mix.Tasks.Pandoc do
  use Mix.Task

  @ext ".md"

  @impl true
  def run(args) do
    switches = [runtime_config: :boolean]
    {opts, remaining_args} = OptionParser.parse_head!(args, switches: switches)

    if opts[:runtime_config] do
      Mix.Task.run("app.config")
    else
      Mix.Task.run("loadpaths")
      Application.ensure_all_started(:pandoc)
    end

    Mix.Task.reenable("pandoc")
    install_and_run(remaining_args)
  end

  defp install_and_run([profile | _args] = all) do
    IO.puts("Converting markdown...")

    profile = String.to_atom(profile)
    config = Application.get_env(:pandoc, profile)
    args = config[:args] || []
    opts = [cd: config[:cd] || File.cwd!()]

    out_path = List.last(args)
    full_out_path = [opts[:cd], out_path] |> Path.join() |> Path.expand()
    File.rm_rf!(full_out_path)
    File.mkdir_p!(full_out_path)

    opts[:cd]
    |> Path.join("*#{@ext}")
    |> Path.wildcard()
    |> Enum.each(fn path ->
      case Pandoc.run(profile, path) do
        0 -> :ok
        status -> Mix.raise("`mix pandoc #{Enum.join(all, " ")}` exited with #{status}")
      end
    end)
  end

  defp install_and_run([]) do
    Mix.raise("`mix pandoc` expects the profile as argument")
  end
end
